/* OpenSSH installation script                                       */
/*                                                                   */
/* Place the following files in this directory:                      */
/* 1. OpenSSH distribution file, rename it to openssh.zip            */
/* 2. Vioroute package, rename it to vioroute.zip                    */
/* 3. Sses.sys from Security/2                                       */
/* 4. Ssesctl.exe from Security/2                                    */
/* 3 and 4 are part of the Security/2 installation. You can copy     */
/* them from a computer where Security/2 has already been installed. */ 
/*                                                                   */
/* 29.05.2004: created                                               */

parse arg target source mode

/* get additional parameters from environment */
product_log       = value("PRODUCT_LOG"      , ,"OS2ENVIRONMENT")
inst_dir          = value("INST_DIR"         , ,"OS2ENVIRONMENT")

/* files/dirs */
cfgsys = target'\CONFIG.SYS'
etc = value('ETC',,'os2environment')
mptnbin = translate(value('PATH',,'os2environment'))
mptnbin = substr(mptnbin, pos(':\MPTN\BIN',mptnbin)-1, 11)
mptndll = substr(mptnbin, 1, 8)'DLL'
user = translate(value('USER',,'os2environment'))
if user  = '' then user = 'os2root'
home = translate(value('HOME',,'os2environment'))
if home  = '' then home = target'\home\'user

/* check if we are in uninstall mode */
if mode = 'UNINSTALL' then do
	call uninstall
	exit
end

/* exit if package does not exist */
if stream(inst_dir'\openssh.zip', 'c', 'query exists') = '' then exit 9

Say 
Say 'Installing SSH/SSHD + some files of Security/2 on your system.'

/* create config */
'@mkdir 'etc'\SSH >nul 2>>&1'
'@unzip -ojC 'inst_dir'\openssh.zip etc\ssh\* -d 'etc'\ssh\. >> 'product_log
'@unzip -ojC 'inst_dir'\vioroute.zip term.cfg -d 'etc'\ssh\. >> 'product_log
Say
Say 'On which port would you like to run sshd? (press enter for default 22)'
pull port
if port = '' then port = 22
sshdcfg = etc'\ssh\sshd_config' 
'@copy 'sshdcfg' 'etc'\ssh\sshd_config.org' 
'@echo Port 'port' > 'sshdcfg
'@echo Protocol 2  >> 'sshdcfg
'@echo HostKey 'etc'\ssh\ssh_host_dsa_key >> 'sshdcfg
'@echo LoginGraceTime 300  >> 'sshdcfg
'@echo Banner 'etc'\ssh_banner.txt  >> 'sshdcfg
'@echo IgnoreUserKnownHosts yes  >> 'sshdcfg
'@echo PasswordAuthentication yes  >> 'sshdcfg
'@echo SyslogFacility AUTH >> 'sshdcfg
'@echo LogLevel INFO >> 'sshdcfg
'@echo PermitRootLogin yes >> 'sshdcfg
'@echo StrictModes yes >> 'sshdcfg
'@echo RSAAuthentication no >> 'sshdcfg
'@echo PubkeyAuthentication yes >> 'sshdcfg
'@echo AuthorizedKeysFile .ssh/authorized_keys >> 'sshdcfg
'@echo IgnoreRhosts yes >> 'sshdcfg
'@echo RhostsRSAAuthentication no >> 'sshdcfg
'@echo HostbasedAuthentication no >> 'sshdcfg
'@echo PermitEmptyPasswords no >> 'sshdcfg
'@echo ChallengeResponseAuthentication no >> 'sshdcfg
'@echo MaxStartups 10 >> 'sshdcfg
'@echo ClientAliveInterval 600 >> 'sshdcfg
'@echo ClientAliveCountMax 3 >> 'sshdcfg
'@echo Subsystem sftp 'mptnbin'\sftp-server.exe >> 'sshdcfg
'@echo Welcome to this secure OS/2 system >> 'etc'\ssh_banner.txt'

/* copy binaries */
'@unzip -ojC 'inst_dir'\openssh.zip  *.exe -d 'mptnbin'\. >> 'product_log
'@unzip -ojC 'inst_dir'\vioroute.zip *.exe -d 'mptnbin'\. >> 'product_log
'@unzip -ojC 'inst_dir'\openssh.zip  *.dll -d 'mptndll'\. >> 'product_log
'@unzip -ojC 'inst_dir'\vioroute.zip *.dll -d 'mptndll'\. >> 'product_log
'@unzip -ojC 'inst_dir'\openssh.zip  *.inf -d 'target'\os2\book\. >> 'product_log

/* create homedir */
'@mkdir 'target'\home >nul 2>>&1'
'@mkdir 'home' >nul 2>>&1'
'@mkdir 'home'\.ssh >nul 2>>&1'

/* copy/create sec/2 files */
'@copy 'inst_dir'\sses.sys 'target'\os2\boot\.'
'@copy 'inst_dir'\ssesctl.exe 'target'\os2\.'
aclcfg = etc'\acls' 
'@echo [100:first] > 'aclcfg
'@echo * * >> 'aclcfg
'@echo cdrw ^<acl^> >> 'aclcfg
'@echo cdrwx ^<user^> >> 'aclcfg
'@echo x ^<dll^> >> 'aclcfg
'@echo+ >> 'aclcfg
'@echo [101:last] >> 'aclcfg
'@echo * * >> 'aclcfg
'@echo 'user':$1$K2hF$WwLkg9GMZjx3L814A4dPj/:0:0::$'space(translate(home,'',':'),0)':$'space(translate(mptnbin,'',':'),0)'\viostart.EXE:$'space(translate(mptnbin,'',':'),0)'\viostart.EXE: > 'etc'\passwd'

/* update config.sys */
do while lines(cfgsys)
	l.q = linein(cfgsys)
	/* check */
	l = translate(l.q)
	if pos('SSES.SYS',    l) > 0 & substr(l, 1, 3) <> 'REM' then found.sys = 1
	if pos('SSESCTL.EXE', l) > 0 & substr(l, 1, 3) <> 'REM' then found.exe = 1
end
call lineout cfgsys
if found.sys = 0 | found.exe = 0 then do
	'@copy 'cfgsys target'\os2\install\config.ssh >> 'product_log 
	if found.sys = 0 then '@echo BASEDEV=SSES.SYS /suanyuser >> 'cfgsys
	if found.exe = 0 then '@echo RUN='target'\os2\ssesctl.exe --source 'etc'\passwd?'etc'\acls --logfile 'target'\ssesctl.log >> 'cfgsys
end

/* create keys */
say
say 'Please type in a passphrase for the root user (press enter for none)'
pull phrase
say
say 'Generating user keys...'
'@'mptnbin'\ssh-keygen.exe -b 2048 -t dsa -f 'home'\.ssh\id_dsa -N "'phrase'"'
'@copy 'home'\.ssh\id_dsa.pub 'home'\.ssh\authorized_keys'
say
say 'Generating server keys...'
'@'mptnbin'\ssh-keygen.exe -b 2048 -t dsa -f 'etc'\ssh\ssh_host_dsa_key -N ""'

/* inform */
say
say 'Root user is os2root, pwd rootos2, see "user.exe -?" to change it.'
say 'Start ssh  daemon with the command "start sshd"'
say 'Start ssh  client with the command "ssh -l 'user' <hostname>"'
say 'Start sftp client with the command "sftp 'user'@<hostname>"'
say 
say 'Press enter to continue.'
pull phrase

exit

uninstall:

	/* update config */
	q = 1
	do while lines(cfgsys)
		l.q = linein(cfgsys)
		if pos('SSES.SYS',    translate(l.q)) > 0 then iterate
		if pos('SSESCTL.EXE', translate(l.q)) > 0 then iterate
		q=q+1
	end
	call lineout cfgsys
	l.0=q-1
	'@copy 'cfgsys target'\os2\install\config.jjc >> 'product_log 
	'del 'cfgsys' >> 'product_log 
	do q=1 to l.0
		call lineout cfgsys, l.q
	end
	call lineout cfgsys

	/* delete files */
	call deldir etc'\ssh'
	'@del 'mptnbin'\ssh*.exe          >> 'product_log
	'@del 'mptnbin'\scp*.exe          >> 'product_log
	'@del 'mptnbin'\sftp*.exe         >> 'product_log
	'@del 'mptnbin'\vioroute*.exe     >> 'product_log
	'@del 'mptndll'\vioroute.dll      >> 'product_log
	'@del 'mptndll'\userctl.dll       >> 'product_log
	'@del 'target'\os2\book\*ssh*.inf >> 'product_log
	'@del 'target'\os2\boot\sses.sys  >> 'product_log
	'@del 'target'\os2\ssesctl.exe    >> 'product_log

return

DelDir: procedure

	parse upper arg Directory
	DirSpec = Directory'\*'

	/* delete subdirectories */
	rc = SysFileTree(DirSpec, Subdirs, 'DO', '*****', '-*---')
	do i = 1 to Subdirs.0
   		call DelDir Subdirs.i
	end

	/* delete files */
	rc = SysFileTree(DirSpec, Files, 'FO', '*****', '-*---')
	do i = 1 to Files.0
		'@del "'Files.i'"'
	end

	/* delete directory */
	'@rmdir "'Directory'"'

return
